@@ -166,8 +166,7 @@ gtk_container_class_init (GtkContainerClass *class)
                     object_class->type,
                     GTK_SIGNAL_OFFSET (GtkContainerClass, need_resize),
                     gtk_container_marshal_signal_4,
-		    GTK_TYPE_BOOL, 0,
-                    GTK_TYPE_WIDGET);
+		    GTK_TYPE_BOOL, 0);
   container_signals[FOREACH] =
     gtk_signal_new ("foreach",
                     GTK_RUN_FIRST,
@@ -309,6 +308,11 @@ void
 gtk_container_add (GtkContainer *container,
 		   GtkWidget    *widget)
 {
+  g_return_if_fail (container != NULL);
+  g_return_if_fail (GTK_IS_CONTAINER (container));
+  g_return_if_fail (widget != NULL);
+  g_return_if_fail (GTK_IS_WIDGET (widget));
+
   gtk_signal_emit (GTK_OBJECT (container), container_signals[ADD], widget);
 }
 
@@ -318,7 +322,10 @@ gtk_container_remove (GtkContainer *container,
 {
   g_return_if_fail (container != NULL);
   g_return_if_fail (GTK_IS_CONTAINER (container));
-
+  g_return_if_fail (widget != NULL);
+  g_return_if_fail (GTK_IS_WIDGET (widget));
+  g_return_if_fail (GTK_CONTAINER (widget->parent) == container);
+  
   if (container->focus_child == widget)
     container->focus_child = NULL;
 
