@@ -276,7 +276,7 @@ gtk_menu_popup (GtkMenu             *menu,
 		GtkWidget           *parent_menu_item,
 		GtkMenuPositionFunc  func,
 		gpointer             data,
-		gint                 button,
+		guint                button,
 		guint32              activate_time)
 {
   g_return_if_fail (menu != NULL);
@@ -351,7 +351,7 @@ gtk_menu_get_active (GtkMenu *menu)
 
 void
 gtk_menu_set_active (GtkMenu *menu,
-		     gint     index)
+		     guint    index)
 {
   GtkWidget *child;
   GList *tmp_list;
@@ -847,18 +847,18 @@ gtk_menu_need_resize (GtkContainer *container)
 static void
 gtk_menu_deactivate (GtkMenuShell *menu_shell)
 {
-  GtkMenuShell *parent;
+  GtkWidget *parent;
 
   g_return_if_fail (menu_shell != NULL);
   g_return_if_fail (GTK_IS_MENU (menu_shell));
 
-  parent = GTK_MENU_SHELL (menu_shell->parent_menu_shell);
-
+  parent = menu_shell->parent_menu_shell;
+  
   menu_shell->activate_time = 0;
   gtk_menu_popdown (GTK_MENU (menu_shell));
 
   if (parent)
-    gtk_menu_shell_deactivate (parent);
+    gtk_menu_shell_deactivate (GTK_MENU_SHELL (parent));
 }
 
 
